﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.ComponentModel;

namespace ThreadedApplication
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();
            InitializeWorker();
        }

        #region UI events

        private void StartButton_Click(object sender, RoutedEventArgs e)
        {
            // If worker is Busy
            if (_worker.IsBusy)
            {
                // stop calculation by requesting for cancellation
                _worker.CancelAsync();

                // Disable the button until the process is stopped
                StartButton.IsEnabled = false;
            }
            else
            {
                // start afresh

                // Change the button content to stop
                StartButton.Content = "Stop";

                // start the thread to calculate prime numbes
                _worker.RunWorkerAsync();
            }
        }
        #endregion

        #region Threading
        // A instance of BackgroundWorker to perfrom calculations
        BackgroundWorker _worker = new BackgroundWorker();

        /// <summary>
        /// Perfroms settings required for running a background thread
        /// </summary>
        void InitializeWorker()
        {
            _worker.WorkerSupportsCancellation = true;
            _worker.WorkerReportsProgress = true;

            _worker.DoWork += new DoWorkEventHandler(_worker_DoWork);
            _worker.ProgressChanged += new ProgressChangedEventHandler(_worker_ProgressChanged);
            _worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(_worker_RunWorkerCompleted);
        }

        /// <summary>
        /// Handler to manage tasks after the worker is completed
        /// this event is fired on cancel, error or when the thread completes.
        /// </summary>
        void _worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            Dispatcher.BeginInvoke(new Action<RunWorkerCompletedEventArgs>(ManageProcessComplete), e);
        }

        /// <summary>
        /// Handler to manage progress, this is usefull to perfrom changes to UI
        /// during calculations
        /// </summary>
        void _worker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            Dispatcher.BeginInvoke(new Action<ProgressChangedEventArgs>(ManageProgress), e);
        }

        /// <summary>
        /// In this handler we perform calculations
        /// </summary>
        void _worker_DoWork(object sender, DoWorkEventArgs e)
        {
            CalculatePrimes(e);
        }

        /// <summary>
        /// Algorithm to calculate primes
        /// </summary>
        void CalculatePrimes(DoWorkEventArgs e)
        {
            // A List to store primes
            List<int> primes=new List<int>();

            // store the first prime 
            primes.Add(2);

            // indicate that the first prime is stored
            _worker.ReportProgress(0, primes[0].ToString());

            // Start checking for primes from 3
            int next = 3;

            // A stopping point for the loop
            while (next < MAX_PRIME)
            {
                // flag to check divisibility
                bool divisible=false;

                // check for prime number
                for (int i = 0; i<primes.Count; i++)
                {
                    // Here we check for cancellation request from UI
                    if (_worker.CancellationPending)
                    {
                        // Cancelling the thread
                        e.Cancel = true;
                        return;
                    }

                    // check for divisibility
                    if (next % primes[i] == 0)
                    {
                        divisible = true;
                        break;
                    }
                }

                // If not divisible then "next" contains a prime number
                if (!divisible)
                {
                    // save the current prime number
                    primes.Add(next);

                    int percent = (int)((((double)next) * 100.0) / ((double)MAX_PRIME));
                    // Report the progress to UI
                    _worker.ReportProgress(Math.Max(0, percent-1), next.ToString());
                }

                next += 2;
            }

            // report end of thread to UI
            _worker.ReportProgress(100, primes.Last().ToString() +" Done");

            // Not required, just clearing the list before quitting
            primes.Clear();
        }

        /// <summary>
        /// Function to manage UI changes after the thread is complete, or error or due to cancellation
        /// </summary>
        void ManageProcessComplete(RunWorkerCompletedEventArgs e)
        {
            // Enable the button, the button was disabled while stopping the process
            StartButton.IsEnabled = true;

            // Indicate to the user that he can start a new process
            StartButton.Content = "Start";

            // Manage cancellations or error
            if(e.Cancelled)
                CurrentPrime.Text += " Cancelled";
            else if (e.Error!=null)
                CurrentPrime.Text = "Error";

            // Process is complete so show that in the progress bar
            CurrentProgress.Value = 100;
        }

        /// <summary>
        /// Function to manage UI changes while the thread is running
        /// </summary>
        void ManageProgress(ProgressChangedEventArgs e)
        {
            CurrentPrime.Text = e.UserState.ToString();
            CurrentProgress.Value = e.ProgressPercentage;
        }
        #endregion

        #region Variables
        const int MAX_PRIME = 1000000;
        #endregion
    }
}
